-- -       -.

--------------------------------------------------------- 
--	-   
--------------------------------------------------------- 
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[FIsLeapYear]') AND type in (N'FN', N'IF', N'TF', N'FS', N'FT'))
	DROP FUNCTION [dbo].[FIsLeapYear]
GO

CREATE FUNCTION [dbo].[FIsLeapYear]
(@p_year SMALLINT)
RETURNS BIT
AS
BEGIN
    DECLARE @p_leap_date SMALLDATETIME
    DECLARE @p_check_day TINYINT
 
    SET @p_leap_date = CONVERT(VARCHAR(4), @p_year) + '0228'
    SET @p_check_day = DATEPART(d, DATEADD(d, 1, @p_leap_date))
    IF (@p_check_day = 29)
        RETURN 1

    RETURN 0  
END
GO

--------------------------------------------------------- 
--	- ,     
--------------------------------------------------------- 
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[FIsWorkDay]') AND type in (N'FN', N'IF', N'TF', N'FS', N'FT'))
	DROP FUNCTION [dbo].[FIsWorkDay]
GO

CREATE FUNCTION [dbo].[FIsWorkDay](
	@Date datetime,
	@CalendarID uniqueidentifier
	)
	RETURNS bit
AS
BEGIN
	DECLARE @Ret bit
	SET @Ret=0
	
	DECLARE @Type int
	SET @Type=-1
	
	DECLARE @Day int
	DECLARE @Year int
	
	SELECT @Day=DATEPART (DY,@Date)
	SELECT @Year=DATEPART (YYYY,@Date)
	
	DECLARE @Shift int
	SET @Shift=0
	

	IF ((SELECT [dbo].[FIsLeapYear] (@Year))=0)
	BEGIN
		IF (@Day>59)
			BEGIN
				SELECT @Shift=1
			END
	END

	SELECT @Type=WDays.[Type] FROM [dvtable_{F12C1136-B351-4037-9DC9-21C042A238AF}] AS WDays
		JOIN [dvtable_{D8B0DEB3-FAE7-4C06-8728-B495985183C9}] AS WYears
			ON WDays.ParentRowID=WYears.RowID
		WHERE	WDays.InstanceID=@CalendarID
			AND	WYears.[Year]=@Year
			AND WDays.DayNumber=@Day
			
	SELECT @Type=WDays.[Type] FROM [dvtable_{F12C1136-B351-4037-9DC9-21C042A238AF}] AS WDays
		JOIN [dvtable_{D8B0DEB3-FAE7-4C06-8728-B495985183C9}] AS WYears
			ON WDays.ParentRowID=WYears.RowID
		WHERE	WDays.InstanceID=@CalendarID
			AND	WYears.[Year]=1796
			AND WDays.DayNumber=@Day+@Shift
	
	IF @Type=-1
			BEGIN
				IF DATEPART (dw,@Date)>5
					BEGIN
						SELECT @Ret=0
					END
					ELSE
					BEGIN
						SELECT @Ret=1
					END
			END
			
	IF @Type=0
		BEGIN
			SELECT @Ret=1
		END
		
	IF @Type>0
		BEGIN
			SELECT @Ret=0
		END
	
	RETURN @Ret		
END

GO

--------------------------------------------------------- 
--	-      
--------------------------------------------------------- 

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[FGetDayDuration]') AND type in (N'FN', N'IF', N'TF', N'FS', N'FT'))
	DROP FUNCTION [dbo].[FGetDayDuration]
GO

CREATE FUNCTION [dbo].[FGetDayDuration](
	@StartTime datetime, 
	@EndTime datetime, 
	@Date datetime,
	@CalendarID uniqueidentifier
	)
	RETURNS int
AS
BEGIN
	DECLARE @Duration int
	SELECT @Duration=0
	
	DECLARE @Day int
	DECLARE @Year int
	
	SELECT @Day=DATEPART(dy,@Date)
	SELECT @Year=DATEPART(yyyy,@Date)
	
	DECLARE @Shift int --    
	SET @Shift=0
	


	SELECT @StartTime=CONVERT(varchar(8), @StartTime,108)
	SELECT @EndTime=CONVERT(varchar(8), @EndTime,108)

	IF ((SELECT [dbo].[FIsLeapYear] (@Year))=0)
	BEGIN
		IF (@Day>59)
			BEGIN
				SELECT @Shift=1
			END
	END
		
	-- ,    
	DECLARE @IsWorkDay bit
	SELECT @IsWorkDay = [dbo].[FIsWorkDay](@Date,@CalendarID)
		IF  @IsWorkDay = 0
			BEGIN
				RETURN @Duration	--  ,  0
			END
	
	
	--    		
	DECLARE @Intervals table
	(
		Position int IDENTITY,
		StartTime datetime,
		EndTime datetime,
		DiffHours int
	)
	
	--     
	DECLARE @SettingsFound int
	SELECT @SettingsFound=0
	INSERT INTO @Intervals
		SELECT CONVERT(varchar(8), StartTime,108),CONVERT(varchar(8), EndTime,108), DATEDIFF(mi,CONVERT(varchar(8), StartTime,108),CONVERT(varchar(8), EndTime,108)) FROM [dvtable_{BF6C7A87-F8F5-4707-98E0-A33E10AE7EF2}] AS WHours
			JOIN [dvtable_{F12C1136-B351-4037-9DC9-21C042A238AF}] AS WDays
				ON WHours.ParentRowID=WDays.RowID
			JOIN [dvtable_{D8B0DEB3-FAE7-4C06-8728-B495985183C9}] AS WYears
				ON WDays.ParentRowID=WYears.RowID
			WHERE	WHours.InstanceID=@CalendarID
				AND	WYears.Year=@Year
				AND WDays.DayNumber=@Day
				AND WDays.[Type]=0
			ORDER BY WHours.StartTime ASC
			
	SELECT @SettingsFound=COUNT(*) FROM @Intervals
	IF @SettingsFound=0		--     
		BEGIN
			INSERT INTO @Intervals
				SELECT CONVERT(varchar(8), StartTime,108),CONVERT(varchar(8), EndTime,108), DATEDIFF(mi,CONVERT(varchar(8), StartTime,108),CONVERT(varchar(8), EndTime,108)) FROM [dvtable_{BF6C7A87-F8F5-4707-98E0-A33E10AE7EF2}] AS WHours
					JOIN [dvtable_{F12C1136-B351-4037-9DC9-21C042A238AF}] AS WDays
						ON WHours.ParentRowID=WDays.RowID
					JOIN [dvtable_{D8B0DEB3-FAE7-4C06-8728-B495985183C9}] AS WYears
						ON WDays.ParentRowID=WYears.RowID
					WHERE	WHours.InstanceID=@CalendarID
						AND	WYears.Year=1796
						AND WDays.DayNumber=@Day+@Shift
						AND WDays.[Type]=0
					ORDER BY WHours.StartTime ASC
		END
		
	SELECT @SettingsFound=COUNT(*) FROM @Intervals
	IF @SettingsFound=0		--     
		BEGIN
				INSERT INTO @Intervals
					SELECT CONVERT(varchar(8), StartTime,108),CONVERT(varchar(8), EndTime,108),DATEDIFF(mi,CONVERT(varchar(8), StartTime,108),CONVERT(varchar(8), EndTime,108)) FROM [dvtable_{6A95F4B5-E86D-4464-99DA-BAFF7264453C}] AS CDef
						WHERE CDef.InstanceID=@CalendarID
						ORDER BY StartTime ASC
		END
		
	SELECT @SettingsFound=COUNT(*) FROM @Intervals
	IF @SettingsFound=0		--    
		BEGIN
			INSERT INTO @Intervals VALUES ('09:00:00','13:00:00',240)
			INSERT INTO @Intervals VALUES ('14:00:00','18:00:00',240)
		END
	
	DECLARE @StartIndex int
	DECLARE @EndIndex int
	DECLARE @PositionMin int
	DECLARE @PositionMax int
	
	SELECT @PositionMin=MIN(Position) FROM @Intervals
	SELECT @PositionMax=MAX(Position) FROM @Intervals
	
	-- ,       
	IF ((SELECT StartTime FROM @Intervals WHERE Position=@PositionMin)>@EndTime)
		BEGIN
			RETURN 0
		END
	IF ((SELECT EndTime FROM @Intervals WHERE Position=@PositionMax)<@StartTime)
		BEGIN
			RETURN 0
		END
	
	--  
	SELECT @StartIndex=Position FROM @Intervals
		WHERE @StartTime>=StartTime AND @StartTime<=EndTime

	SELECT @EndIndex=Position FROM @Intervals
		WHERE @EndTime>=StartTime AND @EndTime<=EndTime
		
	IF 	@StartIndex IS NULL	--       .  
		BEGIN
			SELECT @StartIndex=MAX(Position)+1 FROM @Intervals
				WHERE EndTime<@StartTime AND StartTime<@StartTime
				
			IF @StartIndex IS NULL --      NULL,    
				BEGIN
					SELECT @StartIndex=@PositionMin
				END
				
			SELECT @StartTime=StartTime FROM @Intervals WHERE Position=@StartIndex
		END
		
	IF 	@EndIndex IS NULL	--       .  
		BEGIN
			SELECT @EndIndex=MIN(Position)-1 FROM @Intervals
				WHERE EndTime>@EndTime AND StartTime>@EndTime
			
			IF @EndIndex IS NULL --      NULL,    
				BEGIN
					SELECT @EndIndex=@PositionMax
				END
				
			SELECT @EndTime=EndTime FROM @Intervals WHERE Position=@EndIndex
			
		END
		
	--    
	IF @StartTime>@EndTime
		BEGIN
			RETURN 0
		END
		
	DECLARE @DiffSpan int
	DECLARE @DiffStartSpan int
	DECLARE @DiffEndSpan int
	
	SET @DiffSpan =0
	SET @DiffStartSpan=0
	SET @DiffEndSpan =0
	
	SELECT @DiffSpan=SUM(DiffHours) FROM @Intervals
		WHERE Position>@StartIndex AND Position<@EndIndex
	
	IF @StartIndex !=@EndIndex
	BEGIN		--      
		SELECT @DiffStartSpan=DATEDIFF (mi,@StartTime,EndTime) FROM @Intervals
			WHERE Position=@StartIndex
		
		SELECT @DiffEndSpan=DATEDIFF (mi,StartTime,@EndTime) FROM @Intervals
			WHERE Position=@EndIndex
		
		IF @DiffSpan IS NULL
			BEGIN
				SELECT @DiffSpan=0
			END
	END
	ELSE
	BEGIN		--   
		SELECT @DiffSpan=0
		SELECT @DiffEndSpan =0
		SELECT @DiffStartSpan=DATEDIFF (mi,@StartTime,@EndTime) FROM @Intervals
				WHERE Position=@StartIndex --OR Position=@EndIndex


	END
	
	SELECT @Duration=@DiffSpan+@DiffStartSpan+@DiffEndSpan
	
	RETURN @Duration
END

GO

--------------------------------------------------------------------- 
--	-       
--------------------------------------------------------------------- 

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[FBusinessHours2]') AND type in (N'FN', N'IF', N'TF', N'FS', N'FT'))
	DROP FUNCTION [dbo].[FBusinessHours2]
GO

CREATE FUNCTION [dbo].[FBusinessHours2](
	@StartDate datetime, 
	@EndDate datetime, 
	@CalendarID uniqueidentifier,
	@Mode bit)
	-- @Mode -    .
	-- 0 -  
	-- 1 -  
	RETURNS int
AS
BEGIN

  
  	--     NULL,  -   .    NULL  EndDate   
	IF @EndDate IS NULL SELECT  @EndDate=GETDATE()
	
	
	DECLARE @CurrentDate datetime
	
	
	DECLARE @Duration int
	SET @Duration=0
	
	IF DATEPART(dy,@StartDate)=DATEPART(dy,@EndDate) --      
	BEGIN 
		SELECT @Duration=[dbo].[FGetDayDuration](@StartDate,@EndDate,@StartDate,@CalendarID)
	END
	ELSE
	BEGIN 
		--      
		DECLARE @StartDateTMP as datetime
		SET @StartDateTMP=CONVERT(varchar(8), @StartDate, 112) 
	
		SELECT @Duration=@Duration+[dbo].[FGetDayDuration](@StartDate,DATEADD(mi,1439,CAST(@StartDateTMP as datetime)),@StartDate,@CalendarID)

		--         
		DECLARE @EndDateTMP as datetime
		SET @EndDateTMP=CONVERT(varchar(8), @StartDate, 112) 
		
		SELECT @Duration=@Duration+[dbo].[FGetDayDuration](CAST(@EndDateTMP AS datetime),@EndDate,@EndDate,@CalendarID)

				
		--     
		SET @CurrentDate=DATEADD(dd,1,@StartDate)
		WHILE DATEPART(dy,@CurrentDate)<DATEPART(dy,@EndDate)
			BEGIN
				DECLARE @CurrentDateTMP as datetime
				SET @CurrentDateTMP=CONVERT(varchar(8), @StartDate, 112) 
		
				SELECT @Duration=@Duration+[dbo].[FGetDayDuration](@CurrentDateTMP,DATEADD(mi,1439,@CurrentDateTMP),@CurrentDate,@CalendarID)
				SET @CurrentDate=DATEADD(dd,1,@CurrentDate)
			END	
	END
	
	IF @Mode=1
		BEGIN
			--   
			SELECT @Duration=CAST(ROUND(CAST(@Duration as float)/60,0) as int)
		END
    RETURN @Duration
END

GO